<?php

namespace Drupal\Tests\hospital_price_transparency\Functional;

use Drupal\hospital_price_transparency\Entity\HospitalPriceTransparency;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests hospital_price_transparency module.
 *
 * @group hospital_price_transparency
 */
class HospitalPriceTransparencyTest extends BrowserTestBase {

  /**
   * Allow invalid schema to support date formatter in D9.5.
   *
   * @todo: Remove when we no longer support D9 at all.
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['hospital_price_transparency'];

  /**
   * {@inheritdoc}
   */
  public $defaultTheme = 'stark';

  /**
   * Test essential functionality of hpt entities.
   */
  public function testHospitalPriceTransparency() {
    $this->drupalLogin($this->drupalCreateUser([], 'radmin', TRUE));

    // Make a file.
    $contents = file_get_contents(\Drupal::service('extension.list.module')->getPath('hospital_price_transparency') . '/tests/files/file.csv');
    $file1 = \Drupal::service('file.repository')->writeData($contents, "public://file.csv");

    // Make an hpt that uses this file.
    $hpt1 = HospitalPriceTransparency::create([
      'label' => 'My Test HPT',
      'hospital_name' => 'BurnsMemorial',
      'ein' => '123456789',
      'fid' => $file1->id(),
      'status' => TRUE,
    ]);
    $hpt1->save();

    // View the hpt.
    $this->drupalGet('/hpt/1');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('copper,silicon,vanadium');
    $this->assertSession()->responseHeaderEquals('Content-Type', 'text/csv; charset=UTF-8');
    // View it at its alias.
    $this->drupalGet('/hpt/1/123456789_BurnsMemorial_standardcharges.csv');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('copper,silicon,vanadium');
    $this->assertSession()->responseHeaderEquals('Content-Type', 'text/csv; charset=UTF-8');

    // Update the hpt.
    $hpt1->set('ein','987654321');
    $hpt1->save();
    // View hpt.
    $this->drupalGet('/hpt/1');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('copper,silicon,vanadium');
    // View it at its old alias.
    $this->drupalGet('/hpt/1/123456789_BurnsMemorial_standardcharges.csv');
    $this->assertSession()->statusCodeEquals(404);
    // View it at its alias.
    $this->drupalGet('/hpt/1/987654321_BurnsMemorial_standardcharges.csv');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('copper,silicon,vanadium');

    // Create a new hpt.
    $contents = file_get_contents(\Drupal::service('extension.list.module')->getPath('hospital_price_transparency') . '/tests/files/file.json');
    $file2 = \Drupal::service('file.repository')->writeData($contents, "public://file.json");
    $hpt2 = HospitalPriceTransparency::create([
      'label' => 'My Json HPT',
      'hospital_name' => 'JsonRegional',
      'ein' => '999999999',
      'fid' => $file2->id(),
      'status' => TRUE,
    ]);
    $hpt2->save();

    // View it.
    $this->drupalGet('/hpt/2');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('my json file');
    $this->assertSession()->responseHeaderEquals('Content-Type', 'application/json');
    // Delete the file and confirm we 404.
    $file2->delete();
    $this->drupalGet('/hpt/2');
    $this->assertSession()->statusCodeEquals(404);

    // View the first hpt as anonymous.
    $this->drupalLogin($this->drupalCreateUser(['access content']));
    $this->drupalGet('/hpt/1');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('copper,silicon,vanadium');
    $this->drupalGet('/hpt/1/987654321_BurnsMemorial_standardcharges.csv');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->responseContains('copper,silicon,vanadium');

    // Confirm we can't access various routes.
    $this->drupalGet('/hpt/1/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/hpt/1/delete');
    $this->assertSession()->statusCodeEquals(403);

    // Unpublish the hpt.
    $hpt1->set('status', FALSE);
    $hpt1->save();

    // Confirm anonymous can no longer view hpt.
    $this->drupalGet('/hpt/1');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/hpt/1/987654321_BurnsMemorial_standardcharges.csv');
    $this->assertSession()->statusCodeEquals(403);
  }

}
